﻿using Infragistics.Shared;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinMaskedEdit;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp19
{
    public partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var spinEditorButton = new SpinEditorButton();
            this.ultraDateTimeEditor1.ButtonsRight.Add(spinEditorButton);            
            this.ultraDateTimeEditor1.Visible = false;
            var dt = DummyDataCreator.GetData(10, new Type[] { typeof(string), typeof(DateTime) });            

            // This spin buttons work fine with random dates, but it makes sense to use times that are already in increments of 15 minutes. 
            int minutes = -15;
            foreach (DataRow row in dt.Rows)
            {
                minutes += 15;
                if (minutes >= 60)
                    minutes = 0;

                var dateTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, minutes, 0);
                row["DateTime 1"] = dateTime;
            }
            dt.AcceptChanges();


            this.ultraGrid1.DataSource = dt;

            this.ultraGrid1.KeyDown += this.UltraGrid1_KeyDown;
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            var layout = e.Layout;
            var band = layout.Bands[0];
            var ov = layout.Override;

            var dateTimeColumn = band.Columns["DateTime 1"];
            dateTimeColumn.MaskInput = "{time}";
            dateTimeColumn.EditorComponent = this.ultraDateTimeEditor1;
            dateTimeColumn.ButtonDisplayStyle = Infragistics.Win.UltraWinGrid.ButtonDisplayStyle.Always;

            dateTimeColumn.PerformAutoResize(PerformAutoSizeType.AllRowsInBand, AutoResizeColumnWidthOptions.All);            
        }

        private void UltraGrid1_KeyDown(object sender, KeyEventArgs e)
        {
            var grid = (UltraGrid)sender;
            var activeCell = grid.ActiveCell;
            if (null == activeCell)
                return;

            if (activeCell.Column.EditorComponent != this.ultraDateTimeEditor1)
                return;

            switch (e.KeyCode)
            {
                case Keys.Up:
                    this.Spin(activeCell, true);
                    e.Handled = true;
                    break;
                case Keys.Down:
                    this.Spin(activeCell, false);
                    e.Handled = true;
                    break;

            }
        }

        private void ultraDateTimeEditor1_EditorSpinButtonClick(object sender, Infragistics.Win.UltraWinEditors.SpinButtonClickEventArgs e)
        {
            var cell = e.Context as UltraGridCell;
            var up = e.ButtonType == SpinButtonItem.NextItem;
            this.Spin(cell, up);
        }

        private void Spin(UltraGridCell cell, bool up)
        {
            var originalDateTime = (DateTime)cell.Value;
            var increment = this.GetSpinIncrementResolved(cell);
            if (null != increment)
            {
                var newDateTime = increment.AdjustDateTime(up, originalDateTime);
                cell.Value = newDateTime;
            }
        }

        internal DateTimeIncrement GetSpinIncrementResolved(UltraGridCell cell)
        {
            var cellEditor = cell.EditorResolved as DateTimeEditor;

            // The Current Section is not exposed publicly, so we need to get it via reflection. 
            // This will, of course, only work if we have full rights and permissions. 
            var bindingFlags = System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance;
            var editInfoProp = cellEditor.GetType().GetProperty("EditInfo", bindingFlags);
            var editInfo = editInfoProp.GetValue(cellEditor);
            var currentSectionProp = editInfo.GetType().GetProperty("CurrentSection", bindingFlags);
            var currentSection = currentSectionProp.GetValue(editInfo) as SectionBase;

            if (currentSection is LiteralSection)
                currentSection = currentSection.PrevEditSection;

            if (currentSection == null)
            {
                var lastEditSectionProp = editInfo.GetType().GetProperty("LastEditSection", bindingFlags);
                var lastEditSection = lastEditSectionProp.GetValue(editInfo) as SectionBase;
                currentSection = lastEditSection;
            }

            if (currentSection != null)
            {
                DateTimeIncrement dateTimeIncrement = new DateTimeIncrement();
                if (currentSection is YearSection)
                    dateTimeIncrement.Years++;
                else if (currentSection is MonthSection)
                    dateTimeIncrement.Months++;
                else if (currentSection is DaySection)
                    dateTimeIncrement.Days++;
                else if (currentSection is HourSection)
                    dateTimeIncrement.Hours++;
                else if (currentSection is MinuteSection)
                    dateTimeIncrement.Minutes+=15;
                else if (currentSection is SecondSection)
                    dateTimeIncrement.Seconds++;
                else
                    return null;

                return dateTimeIncrement;
            }

            return null;
        }
    }
}
